
#ifndef BONDPOTENTIAL_H
#define BONDPOTENTIAL_H

#include "rumd/Potential.h"

class MoleculeData; 

///////////////////////////////////////////////////////
// Abstract BondPotential class
///////////////////////////////////////////////////////

class BondPotential : public Potential{
 public:
  MoleculeData *M;
  unsigned num_bonds;
  unsigned num_threads, num_blocks;
  bool EnergyIncludedInParticleSum() const { return true; };
  void Initialize() {}
};

///////////////////////////////////////////////////////
// Specialized bond potentials
///////////////////////////////////////////////////////

// Harmonic
class BondHarmonic : public BondPotential{
 private:
  BondHarmonic(const BondHarmonic&);
  BondHarmonic& operator=(const BondHarmonic&);

 public:
  BondHarmonic(MoleculeData *);
  void CalcF(bool initialize, bool calc_stresses);
  double GetPotentialEnergy();
};

// FENE
class BondFENE : public BondPotential{
 private:
  BondFENE(const BondFENE&);
  BondFENE& operator=(const BondFENE&);
  
 public:
  BondFENE(MoleculeData *);
  void CalcF(bool initialize, bool calc_stresses);
  double GetPotentialEnergy();
};


///////////////////////////////////////////////////////
// Kernel prototypes
///////////////////////////////////////////////////////

template <int stress, int energy, class Simbox>
  __global__ void Harmonic( float4 *r, float4 *f, float4 *w,  float4 *sts,
			    uint2 *blist, uint1 *btlist, float2 *bplist,
			    float *belist, int1 *btlist_int, float *bonds,
			    unsigned num_bonds, Simbox *simbox, float *simboxpointer );

template <int stress, int energy, class Simbox>
  __global__ void FENE( float4 *r, float4 *f, float4 *w,  float4 *sts,
			uint2 *blist, uint1 *btlist, float2 *bplist,
			float *belist, int1 *btlist_int, float *bonds,
			unsigned num_bonds, Simbox *simbox, float *simboxpointer );

/////////////////////////////////////////////////////////

__global__ void BondResetForces( float4 *f, float4 *w );
__device__ inline void BondAtomicFloatAdd(float *address, float val);

#endif // BONDPOTENTIAL_H
