#ifndef POTENTIAL_H
#define POTENTIAL_H

#include "rumd/rumd_base.h"
#include "rumd/KernelPlan.h"
#include <string>
#include "rumd/RUMD_Error.h"

class Sample; class ParticleData; class SimulationBox;

class Potential{
 private:
  Potential(const Potential&); 
  Potential& operator=(const Potential&); 
  std::string ID_String;
  
 protected:
  Sample* sample;
  KernelPlan kp;
  const ParticleData* particleData;
  SimulationBox* simBox;
  bool verbose;
  
 public:
  Potential();
  virtual ~Potential(){}

  virtual Potential* Copy(){ throw RUMD_Error("Potential","Copy","Not implemented yet"); }  
  virtual void CalcF(bool initialize, bool calc_stresses) = 0;


  // tells whether the contribution to the total energy is included in the poarticle energies
  virtual bool EnergyIncludedInParticleSum() const = 0;
  virtual void UpdateAfterSorting( unsigned*, unsigned* ) {};

  // Set methods.
  virtual void SetSample(Sample* sample);
  // called by SetSample, must be implemented by different potentials
  virtual void Initialize() = 0;

  virtual void SetVerbose(bool vb) { verbose = vb; }
  void SetID_String(const std::string& set_ID_String){ ID_String = set_ID_String; }
  
  // Get methods.
  virtual double GetPotentialEnergy() { return 0.; }
  std::string GetID_String() const { return ID_String; }
};

#endif // POTENTIAL_H
